from copy import deepcopy
from os import environ

import pytest
from requests_ms_auth import MsRequestsSession
from requests_ms_auth.ms_session_config import MsSessionConfig


INTEGRATION_TENANT = environ["INTEGRATION_TENANT"]
INTEGRATION_CLIENT_ID = environ["INTEGRATION_CLIENT_ID"]
INTEGRATION_CLIENT_SECRET = environ["INTEGRATION_CLIENT_SECRET"]
INTEGRATION_RESOURCE = environ["INTEGRATION_RESOURCE"]
INTEGRATION_AUTHORITY_HOST_URL = environ["INTEGRATION_AUTHORITY_HOST_URL"]
INTEGRATION_LIVE_VERIFICATION_URL = environ["INTEGRATION_LIVE_VERIFICATION_URL"]

AUTH = MsSessionConfig(
    resource=INTEGRATION_RESOURCE,
    tenant=INTEGRATION_TENANT,
    authority_host_url=INTEGRATION_AUTHORITY_HOST_URL,
    client_id=INTEGRATION_CLIENT_ID,
    client_secret=INTEGRATION_CLIENT_SECRET,
    do_adal=True,
)


@pytest.fixture(scope="function")
def new_session():
    session_config = deepcopy(AUTH)
    return MsRequestsSession(msrs_auth_config=session_config)



"""Contain logic for Backend Client for using by Session"""
import logging
import time

import oauthlib.oauth2

logger = logging.getLogger(__name__)


class MsBackendApplicationClient(oauthlib.oauth2.BackendApplicationClient):
    """Inherits Client for session requests to add more functionality"""

    def is_access_token_expired(self) -> bool:
        """Check if 'access token' expired

        Note: expiration time will be speed up not to have overlap when token is invalid

        Returns:
            True: if token expired
            False: if token not expired
        """
        token_expiration_time_correction = 15  # in seconds

        if self._expires_at and (self._expires_at - token_expiration_time_correction) < time.time():
            logger.debug("Token has expired.")
            return True
        return False



import logging
from typing import Dict, Optional, Tuple, Union

import adal
import msal
import requests
import requests_oauthlib
import simplejson
from requests.structures import CaseInsensitiveDict

from requests_ms_auth.ms_backend_application_client import MsBackendApplicationClient
from requests_ms_auth.ms_session_config import MsSessionConfig

logger = logging.getLogger(__name__)


class MsRequestsSession(requests_oauthlib.OAuth2Session):
    """A wrapper for OAuth2Session that also implements adal token fetch.

    See https://requests.readthedocs.io/en/latest/_modules/requests/sessions/#Session
    See https://requests-oauthlib.readthedocs.io/en/latest/api.html#oauth-2-0-session
    See https://adal-python.readthedocs.io/en/latest/
    """

    msrs_aouth_header = "Authorization"
    msrs_access_token_name = "access_token"

    def __init__(self, msrs_auth_config: Union[MsSessionConfig, dict]):
        if not isinstance(msrs_auth_config, MsSessionConfig):  # TODO remove this in the future versions
            logger.warning("Using of 'msrs_auth_config' as dict will be deprecated in the next versions.")
            msrs_auth_config = MsSessionConfig(**msrs_auth_config)

        self.msrs_client_id = msrs_auth_config.client_id
        self.msrs_do_adal = msrs_auth_config.do_adal
        self.msrs_client_secret = msrs_auth_config.client_secret
        self.msrs_resource_uri = msrs_auth_config.resource
        self.msrs_tenant = msrs_auth_config.tenant
        self.msrs_validate_authority = self.msrs_tenant != "adfs"
        self.msrs_scope = msrs_auth_config.scope
        self.msrs_verification_url = msrs_auth_config.verification_url
        self.msrs_verification_element = msrs_auth_config.verification_element
        self.msrs_auto_refresh_url = f"{msrs_auth_config.authority_host_url}/{self.msrs_tenant}"
        self.msrs_token = self._fetch_access_token()
        if not self.msrs_token:
            raise Exception("Could not generate 'msrs_token'")
        self.msrs_client = MsBackendApplicationClient(
            client_id=self.msrs_client_id, token=self.msrs_token, scope=self.msrs_scope,
        )
        logging.debug(
            f"__init__(client_id={self.msrs_client_id}, auto_refresh_url={self.msrs_auto_refresh_url}, "
            f"scope={self.msrs_scope})."
        )

        super(MsRequestsSession, self).__init__(client=self.msrs_client, token=self.msrs_token)

        validation_ok, validation_error = self.verify_auth()
        if not validation_ok:
            raise Exception(validation_error)

    def _fetch_access_token(self) -> Dict:
        if self.msrs_do_adal:
            return self._fetch_access_token_adal()
        else:
            return self._fetch_access_token_msal()

    def _fetch_access_token_msal(self) -> Dict:
        self.msrs_ms_token = None
        self.msrs_oathlib_token = None
        try:
            context = msal.ConfidentialClientApplication(
                authority=self.msrs_auto_refresh_url,
                validate_authority=self.msrs_validate_authority,
                client_id=self.msrs_client_id,
                client_credential=self.msrs_client_secret,
            )
            scopes = [f"{self.msrs_resource_uri}/.default"]
            self.msrs_ms_token = context.acquire_token_for_client(scopes=scopes)
            if self.msrs_ms_token:
                if self.msrs_ms_token.get("error"):
                    error = self.msrs_ms_token.get("error")
                    desc = self.msrs_ms_token.get("error_description")
                    raise Exception(f"Error fetching MSAL token ({error}): {desc}")
                self.msrs_oathlib_token = {
                    "access_token": self.msrs_ms_token.get("access_token", ""),
                    "refresh_token": self.msrs_ms_token.get("refresh_token", ""),
                    "token_type": self.msrs_ms_token.get("token_type", "Bearer"),
                    "expires_in": self.msrs_ms_token.get("expires_in", 0),
                    "ext_expires_in": self.msrs_ms_token.get("ext_expires_in", 0),
                }
            else:
                logger.error(f"Could not get token for client {self.msrs_auto_refresh_url}")
                raise Exception("No token acquired")
            if not self.msrs_oathlib_token.get("access_token"):
                logger.warning(f"Token aqcuired seems lacking")
                raise Exception("Token aqcuired seems lacking")
        except Exception as e:
            logger.error(f"Error fetching token: {e}", exc_info=True)
            logger.warning(f"NOTE: {self}")
            raise e
        return self.msrs_oathlib_token

    def _fetch_access_token_adal(self) -> Dict:
        self.msrs_ms_token = None
        self.msrs_oathlib_token = None
        try:
            context = adal.AuthenticationContext(
                authority=self.msrs_auto_refresh_url, validate_authority=self.msrs_validate_authority, api_version=None,
            )
            self.msrs_ms_token = (
                context.acquire_token_with_client_credentials(
                    self.msrs_resource_uri, self.msrs_client_id, self.msrs_client_secret
                )
                or {}
            )
            if self.msrs_ms_token:
                self.msrs_oathlib_token = {
                    self.msrs_access_token_name: self.msrs_ms_token.get("accessToken", ""),
                    "refresh_token": self.msrs_ms_token.get("refreshToken", ""),
                    "token_type": self.msrs_ms_token.get("tokenType", "Bearer"),
                    "expires_in": self.msrs_ms_token.get("expiresIn", 0),
                }
            else:
                logger.error(f"Could not get token for client {self.msrs_auto_refresh_url}")
                raise Exception("No token acquired")
        except Exception as e:
            logger.error(f"Error fetching token: {e}", exc_info=True)
            logger.warning(f"NOTE: {self}")
            raise e
        return self.msrs_oathlib_token

    def verify_auth(self) -> Tuple[bool, Optional[str]]:
        if self.msrs_verification_url:
            logger.debug("Verification URL specified, performing http verification")
            res = self.get(self.msrs_verification_url)
            if res is None:
                return False, "Verification failed: No response object returned"
            if not res:
                try:
                    res.raise_for_status()
                except requests.exceptions.HTTPError:
                    return False, f"Verification failed: Request returned HTTP {res.status_code} ({res.reason})"
            if self.msrs_verification_element:
                logger.debug("Verification element specified, performing json result verification")
                if not res.text:
                    return False, "Verification failed: Request returned empty response"
                j = None
                try:
                    j = res.json()
                except simplejson.errors.JSONDecodeError:  # type: ignore
                    return False, f"Verification failed: Response was not json. Excerpt: '{res.text[0:100]}'..."
                if not j:
                    return False, "Verification failed: Returned json was empty"

                if not j.get(self.msrs_verification_element, False):
                    return (
                        False,
                        f"Verification failed: Expected json element '{self.msrs_verification_element}' not "
                        + f"found in response.  Excerpt: '{res.text[0:100]}...'",
                    )
            else:
                logger.debug(f"No verification element specified, skipping json result verification")
        else:
            logger.debug(f"No verification URL specified, skipping http verification")
        # Success
        return True, None

    def prepare_request(self, request):
        logging.debug(f"prepare_request(method={request.method}, url='{request.url}').")
        return super(MsRequestsSession, self).prepare_request(request)

    def request(
        self, method, url, data=None, headers=None, withhold_token=False, client_id=None, client_secret=None, **kwargs,
    ):
        """This method will be called each time smth should be sent

        Notes:
            if 'sent' method will be called directly, this method will be invoked if there's no 'token' in the headers
            'access token' validity will be checked/renewed as well
        """
        logging.debug(f"request(method={method}, url='{url}').")
        self.access_token_check_and_renew(headers)

        return super(MsRequestsSession, self).request(
            method=method,
            url=url,
            data=data,
            headers=headers,
            withhold_token=withhold_token,
            client_id=client_id,
            client_secret=client_secret,
            **kwargs,
        )

    def send(self, request, **kwargs):
        """Send prepared Request with auth token

        Notes:
            if there's no auth header or it's empty -> use inherited from oauth2 functionality to add token on requests
            'access token' validity will be checked/renewed as well
        """
        logging.debug(f"send(method={request.method}, url='{request.url}').")
        self.access_token_check_and_renew(request.headers)

        try:
            if request.headers is not None and request.headers.get(self.msrs_aouth_header, False):
                # send prepared Request if access token exists in the Request
                response = super().send(request, **kwargs)
            else:
                # make another call to get token into the header
                response = self.request(
                    method=request.method, url=request.url, data=request.body, headers=request.headers, **kwargs,
                )
            logging.debug(f"Response head follows: -----------------------")
            logging.debug(response.content[0:200])
            return response
        except requests.exceptions.NewConnectionError as e:
            logger.error(f"Could not connect (method={request.method}, url='{request.url}'): {e}")
            raise e
        except requests.exceptions.HTTPError as e:
            logger.error(f"HTTP STATUS CODE WAS: {e}")
            raise e
        except Exception as e:
            logger.error(
                f"Could not perform request(method={request.method}, url='{request.url}'): {e}", exc_info=True,
            )
            raise e

    def access_token_check_and_renew(self, headers: Optional[CaseInsensitiveDict] = None) -> None:
        """Check if 'access token' expired and renew it if needed

        Args:
            headers: Request headers. You should pass it by link (not copy) for auth header to be deleted.
                If there's already existing header with old token - we need to delete it fo the future renew.
        """
        if self.msrs_client.is_access_token_expired():
            self.access_token_run_renewal(headers)

    def access_token_run_renewal(self, headers: Optional[CaseInsensitiveDict] = None) -> None:
        """Renew 'access token' for the Session and its Client instances.

        Args:
            headers: Request headers. You should pass it by link (not copy) for auth header to be deleted.
                If there's already existing header with old token - we need to delete it fo the future renew.
        """
        self.msrs_token = self._fetch_access_token()
        if not self.msrs_token:
            raise Exception("Could not fetch token")
        self.msrs_client.access_token = self.msrs_token[self.msrs_access_token_name]
        self.token = self.msrs_token

        try:
            del headers[self.msrs_aouth_header]  # type: ignore
        except (KeyError, TypeError):
            pass

    def close(self):
        logging.debug(f"close().")
        return super(MsRequestsSession, self).close()

    def __repr__(self):
        return f"""{self.__class__.__name__} {{
    type:                 {'ADAL' if self.msrs_do_adal else 'MSAL'}
    client_id:            [hidden]
    resource_uri:         {self.msrs_resource_uri}
    client_secret:        [hidden]
    tenant:               {self.msrs_tenant}
    validate_authority:   {self.msrs_validate_authority}
    auto_refresh_url:     {self.msrs_auto_refresh_url}
    verification:         {self.msrs_verification_url}
        {f"for '{self.msrs_verification_element}'" if self.msrs_verification_element else ''}
}}
"""



import pkg_resources
import os
from .ms_session import MsRequestsSession
from .ms_session_config import MsSessionConfig

__version__ = "0.0.0"

rmsa_path: str = os.path.abspath(os.path.join(os.path.dirname(__file__), "../"))
rmsa_version_file = f"{rmsa_path}/VERSION"


def _read_file(fname, strip=True):
    fn = os.path.join(os.path.dirname(os.path.abspath(__file__)), fname)
    data = ""
    if os.path.exists(fn):
        with open(fn) as f:
            data = f.read()
            data = data.strip() if strip else data
    return data


if pkg_resources.resource_exists(__name__, "VERSION"):
    __version__ = pkg_resources.resource_string(__name__, "VERSION").decode("utf-8").strip()
elif os.path.exists(rmsa_version_file):
    __version__ = _read_file(rmsa_version_file)
else:
    # logger.warning("No version found")
    pass

__all__ = ["MsRequestsSession", "MsSessionConfig", "__version__"]



import logging

import requests
from requests import Response
from tests.integration_tests.conftest import INTEGRATION_LIVE_VERIFICATION_URL

logger = logging.getLogger(__name__)

MSRS_VERIFICATION_ELEMENT = "revision"


def test_session_verification_url(new_session):
    new_session.msrs_verification_url = INTEGRATION_LIVE_VERIFICATION_URL
    new_session.msrs_verification_element = MSRS_VERIFICATION_ELEMENT
    ok, message = new_session.verify_auth()
    assert ok


def test_session_get_method(new_session):
    res = new_session.get(INTEGRATION_LIVE_VERIFICATION_URL)
    assert_success_for_revision(res)


def test_session_send_method(new_session):
    request = requests.Request("GET", INTEGRATION_LIVE_VERIFICATION_URL)
    res = new_session.send(request.prepare())
    assert_success_for_revision(res)


def test_session_token_renew(new_session):
    expire_adjusting_seconds = 3600  # this is 60 minutes (default token expiration time)

    res = new_session.get(INTEGRATION_LIVE_VERIFICATION_URL)
    assert res.status_code == 200

    expired_token = "expired_token"
    new_session.access_token = expired_token
    new_session.msrs_client.access_token = expired_token
    new_session.msrs_client._expires_at -= expire_adjusting_seconds

    res = new_session.get(INTEGRATION_LIVE_VERIFICATION_URL)
    assert new_session.access_token != expired_token
    assert_success_for_revision(res)


def assert_success_for_revision(response: Response):
    """assert 200 response codee and existence of 'revision' in json response"""
    assert response.status_code == 200
    assert MSRS_VERIFICATION_ELEMENT in response.json()



#!/usr/bin/env python

# To run this example, please do the following:
#
# 1. install requirements using the following command:
# python -m pip install --upgrade -r requirements.in
#
# 2. Insert your actual credentials in the time_series_api_credentials.yaml file
#
# 3. Run the script from Pyhton3.7 environment:
# ./time_series_api_example.py

from requests_ms_auth import MsRequestsSession
import logging
import pprint
import yaml

# Set default log level to "INFO"
from requests_ms_auth.ms_session_config import MsSessionConfig

logging.basicConfig(level=logging.INFO)

# Create the logger we will use
logger = logging.getLogger(__file__)

# Main program entry point
if __name__ == "__main__":

    # 1. Load credentials from file (never store credentials in git)!
    filename = "time_series_api_credentials.yaml"
    logger.info(f"Loading credentials from {filename}")
    with open(filename, "r") as stream:
        auth_config = MsSessionConfig(**yaml.safe_load(stream))
    logger.info("Loaded credentials:\n" + pprint.pformat(auth_config))

    # 2.Instanciate a session with authentication dict as parameters
    session = MsRequestsSession(msrs_auth_config=auth_config)
    logger.info("Created session:\n" + str(session))

    # 3. Make a call to time series api using our session
    base_url = "https://api.gateway.equinor.com/plant/timeseries/v1.5"
    body = {"name": "PT-13005/MeasA/PRIM", "assetId": "SFB"}
    res = session.get(base_url, params=body)
    logger.info("Data from time series API:\n" + pprint.pformat(res.json()))



#!/usr/bin/env python

# To run this example, please do the following:
#
# 1. install requirements using the following command:
# python -m pip install --upgrade -r requirements.in
#
# 2. Insert your actual credentials in the gordo_credentials.yaml file
#
# 3. Run the script from Pyhton3.7 environment:
# ./gordo_example.py

from requests_ms_auth import MsRequestsSession
from gordo.client.client import Client
import logging
import pprint
import yaml

# Set default log level to "INFO"
from requests_ms_auth.ms_session_config import MsSessionConfig

logging.basicConfig(level=logging.INFO)

# Create the logger we will use
logger = logging.getLogger(__file__)


# Main program entry point
if __name__ == "__main__":

    # 1. Load credentials from file (never store credentials in git)!
    filename = "time_series_api_credentials.yaml"
    logger.info(f"Loading credentials from {filename}")
    auth_config = {}
    with open(filename, "r") as stream:
        auth_config = MsSessionConfig(**yaml.safe_load(stream))
    logger.info("Loaded credentials:\n" + pprint.pformat(auth_config))

    # 2. Instanciate a session with authentication dict as parameters
    session = MsRequestsSession(auth_config)

    # 3. Instanciate the Gordo client using our session
    client = Client(project="ioc-1901", host="ioc.dev.aurora.equinor.com", session=session)

    # 4. Use Gordo client to make sure it works
    revisions = client.get_revisions()
    logger.info(pprint.pformat(revisions))



"""tests for MsBackendApplicationClient class"""
from time import time as time_now

import pytest
from mock import patch


@pytest.mark.parametrize(
    "increase_to_time_now,expected", [(3586, True), (3575, False), (1, False)],
)
def test_is_access_token_expired(increase_to_time_now: int, expected: bool, backend_client):
    """Explanation: '3586' ~ '59,76667' minutes, so we increase time() with it and check for expiration"""
    with patch("requests_ms_auth.ms_backend_application_client.time.time") as time_mock:
        time_mock.return_value = time_now() + increase_to_time_now  # adjust time() for comparing with '_expire_at'

        res = backend_client.is_access_token_expired()
        assert res is expected



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

import logging

logger = logging.getLogger(__name__)

setup_requirements = ["pytest-runner", "setuptools_scm"]


version = open("VERSION").read().strip()


def remove_comment(line, sep="#"):
    i = line.find(sep)
    if i >= 0:
        line = line[:i]
    return line.strip()


def read_requirements_file(fname: str):
    fn = os.path.join(os.path.dirname(os.path.abspath(__file__)), fname)
    print(f"Reading requirements from {fn}")
    lines = []
    with open(fn) as f:
        for r in f.readlines():
            r = r.strip()
            if len(r) < 1:
                continue
            r = remove_comment(r)
            if len(r) < 1:
                continue
            lines.append(r)
    return lines


setup(
    name="requests_ms_auth",
    version=version,
    packages=find_packages(exclude=["tests*"]),
    setup_requires=setup_requirements,
    license="AGPL-3.0",
    description="Python requests session for microsoft with support for oauth2, adal and msal",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=read_requirements_file("requirements/requirements.in"),  # Allow flexible deps for install
    tests_require=read_requirements_file("requirements/test_requirements.txt"),  # Use rigid deps for testing
    test_suite="tests",
    python_requires=">=3.4.0",
    url="https://github.com/equinor/requests_ms_auth",
    download_url=f"https://github.com/equinor/requests_ms_auth/dist/requests_ms_auth-{version}.tar.gz",
    author="Lennart Rolland",
    author_email="lennartrolland@gmail.com",
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



import pytest

from requests_ms_auth.ms_backend_application_client import MsBackendApplicationClient


@pytest.fixture(scope="function")
def backend_client() -> MsBackendApplicationClient:
    default_expire_in = 3599
    token = {"access_token": "foo", "refresh_token": "", "token_type": "Bearer", "expires_in": default_expire_in}
    return MsBackendApplicationClient(client_id="foo_id", token=token)



from unittest import mock
import adal
import logging
import os
import pytest
import requests_ms_auth
import typing
import json
import requests

logger = logging.getLogger(__name__)


@pytest.fixture
def auth_config_dummy():
    return {
        "resource": "dummy-resource",
        "tenant": "dummy-tenant",
        "authority_host_url": "dummy-authority",
        "client_id": "dummy-client",
        "client_secret": "dummy-secret",
        "verification_url": "https://bob.com",
    }


@pytest.fixture
def auth_config_live_msal():
    with open("secrets.json", "r") as file:
        data = json.load(file)
    return data[0]


@pytest.fixture
def auth_config_live_adal():
    with open("secrets.json", "r") as file:
        data = json.load(file)
    data = data[0]
    data["do_adal"] = True
    return data


# {
# "resource": "dummy-resource",
# "tenant": "dummy-tenant",
# "authority_host_url": "dummy-authority",
# "client_id": "dummy-client",
# "client_secret": "dummy-secret",
# "verification_url": "https://bob.com",
# }


VALID_TOKEN: typing.Dict[str, typing.Union[str, int]] = {
    "accessToken": "dummy-token",
    "refreshToken": "dummy-refresh",
    "tokenType": "dummy-type",
    "expiresIn": 1000,
}
BAD_TOKEN: typing.Dict[str, typing.Union[str, int]] = {}


@mock.patch("adal.AuthenticationContext", autospec=True)
@pytest.mark.parametrize(
    "token,expected_oath_token",
    [
        # When keys provided, remapped to `oath` key strings
        (
            VALID_TOKEN,
            {
                "access_token": "dummy-token",
                "refresh_token": "dummy-refresh",
                "token_type": "dummy-type",
                "expires_in": 1000,
            },
        ),
        # Single key provided, returns `oath` defaults for other keys
        (
            {"accessToken": "dummy-token"},
            {"access_token": "dummy-token", "refresh_token": "", "token_type": "Bearer", "expires_in": 0,},
        ),
        # Empty token provided, get `oath` token of `None`
        (BAD_TOKEN, None),
    ],
)
def todo_test_fetch_access_token_functioning_adal(
    MockAuthenticationContext, auth_config_live_msal, token, expected_oath_token
):
    """
    Test that Adal auth token values result in expected OAuth tokens
    """

    mock_auth_context = MockAuthenticationContext(
        authority="https://dummy-authority", validate_authority=None, api_version=None
    )

    mock_auth_context.acquire_token_with_client_credentials.return_value = token
    session = requests_ms_auth.MsRequestsSession(auth_config_dummy)
    assert session._fetch_access_token() == expected_oath_token


@mock.patch("adal.AuthenticationContext", autospec=True)
def todo_test_fetch_access_token_malfunctioning_adal(MockAuthenticationContext, auth_config_dummy):
    """
    Test that when adal methods error, Exception is raises
    """
    # Context retrieval success, token retrieval method errors
    mock_auth_context = MockAuthenticationContext(
        authority="https://dummy-authority", validate_authority=None, api_version=None
    )
    mock_auth_context.acquire_token_with_client_credentials.side_effect = Exception
    with pytest.raises(Exception):
        fetch_access_token(auth_config_dummy)

    # Context generation errors
    MockAuthenticationContext.side_effect = Exception
    with pytest.raises(Exception):
        fetch_access_token(auth_config_dummy)


@mock.patch("requests_ms_auth.OAuth2Session", autospec=True)
@mock.patch("requests_ms_auth.adal.AuthenticationContext", autospec=True)
def todo_test_create_auth_session(MockAuthenticationContext, MockOAuth2Session, auth_config_dummy):
    """
    Test that OAuth session creation logic
    1. A valid token, and successful session creation returns the session
    2. A valid token, and failed session creation returns `None`.
    3. An invalid token, returns `None`
    """
    mock_auth_context = MockAuthenticationContext(
        authority="https://dummy-authority", validate_authority=None, api_version=None
    )

    # OAuth token retrieved, session created and returns
    MockOAuth2Session.side_effect = "valid-session"
    mock_auth_context.acquire_token_with_client_credentials.return_value = VALID_TOKEN
    assert create_auth_session(auth_config_dummy)

    # OAuth token retrieved, session creation throws exception, return session of `None`
    MockOAuth2Session.side_effect = Exception
    mock_auth_context.acquire_token_with_client_credentials.return_value = BAD_TOKEN
    assert create_auth_session(auth_config_dummy) is None

    # OAuth token not retrieved, return session of `None`
    mock_auth_context.acquire_token_with_client_credentials.return_value = BAD_TOKEN
    assert create_auth_session(auth_config_dummy) is None


# Make sure actually json was returned (HTTP 200 may indicate a web proxy web page being displayed)
def assert_json_response(res, element):
    # logger.warning(res)
    assert res.text
    j = res.json()
    assert j
    # logger.warning(j)
    data = j[element]
    assert data


def todo_test_all(auth_config_live_adal):
    with open("secrets.json", "r") as file:
        data = json.load(file)
    for auth in data:
        verification_url = auth["verification_url"]
        verification_element = auth["verification_element"]
        for do_adal in [True]:
            auth["do_adal"] = do_adal
            session = requests_ms_auth.MsRequestsSession(auth)
            logger.warning(f"Testing {session}")
            # Verification
            ok, message = session.verify_auth()
            assert ok
            logger.warning(f"Message: {message}")
            # Direct
            res = session.get(session.msrs_verification_url)
            assert_json_response(res, verification_element)
            # Prepared
            req = requests.Request("GET", verification_url)
            res = session.send(req.prepare())
            assert_json_response(res, verification_element)



"""Config options for MsRequestsSession class"""
from dataclasses import dataclass, field
from typing import List, Optional


@dataclass
class MsSessionConfig:
    """Dataclass-configuration for creating 'MsRequestsSession'

    Dataclass attributes:
        client_id: client ID
        client_secret: client secret
        verification_url: you service url to call
        verification_element: key in json that should be returned in the response on the first json "level"
        do_adal: if True -> use 'adal' method to fetch access token; otherwise use 'MSAL'
        resource: uri/identifier of the resource to which you're going to make calls to
        authority_host_url: base authority to issuing the token
        tenant: ID of the authority for "authority_host_url" that is responsible for token issuing
        scope: scope of the permissions that will be given
    """

    client_id: str
    client_secret: str
    verification_url: Optional[str] = None
    verification_element: Optional[str] = None
    do_adal: bool = False
    resource: str = "https://management.core.windows.net/"
    authority_host_url: str = "https://login.microsoftonline.com"
    tenant: str = "adfs"
    scope: List[str] = field(default_factory=lambda: ["read", "write"])
